@@ -1,22 +1,19 @@
-from typing import Type
-import asyncio
-
-from solana.publickey import PublicKey
-from solana.transaction import TransactionSignature
-from solana.keypair import Keypair
-from solana.system_program import SYS_PROGRAM_ID
-from solana.sysvar import SYSVAR_RENT_PUBKEY
+from typing import Optional
+from solders.pubkey import Pubkey
+from solders.signature import Signature
+from solders.keypair import Keypair
+from solders.system_program import ID
+from solders.sysvar import RENT
 from spl.token.constants import TOKEN_PROGRAM_ID
 from anchorpy import Program, Provider, Context
 
-from driftpy.clearing_house import (
-    ClearingHouse,
+from driftpy.drift_client import (
+    DriftClient,
 )
 from driftpy.constants.numeric_constants import PEG_PRECISION
-from driftpy.types import OracleGuardRails, OracleSource
+from driftpy.types import OracleGuardRails, OracleSource, PrelaunchOracleParams
 from driftpy.addresses import *
 from driftpy.accounts import get_state_account
-from anchorpy import Wallet
 from driftpy.constants.config import Config
 from anchorpy import Provider, Idl
 import driftpy
@@ -29,46 +26,19 @@ from driftpy.constants.numeric_constants import (
 from driftpy.accounts import get_perp_market_account
 
 
-class Admin(ClearingHouse):
-    @staticmethod
-    def from_config(
-        config: Config,
-        provider: Provider,
-        authority: Keypair = None,
-        admin: bool = False,
-    ):
-        # read the idl
-        file = Path(str(driftpy.__path__[0]) + "/idl/drift.json")
-        with file.open() as f:
-            idl_dict = json.load(f)
-        idl = Idl.from_json(idl_dict)
-
-        # create the program
-        program = Program(
-            idl,
-            config.clearing_house_program_id,
-            provider,
-        )
-
-        clearing_house = Admin(program, authority)
-        clearing_house.config = config
-        clearing_house.idl = idl
-
-        return clearing_house
-
+class Admin(DriftClient):
     async def initialize(
         self,
-        usdc_mint: PublicKey,
+        usdc_mint: Pubkey,
         admin_controls_prices: bool,
-    ) -> tuple[TransactionSignature, TransactionSignature]:
-
+    ) -> tuple[Signature, Signature]:
         state_account_rpc_response = (
             await self.program.provider.connection.get_account_info(
                 get_state_public_key(self.program_id)
             )
         )
-        if state_account_rpc_response["result"]["value"] is not None:
-            raise RuntimeError("Clearing house already initialized")
+        if state_account_rpc_response.value is not None:
+            raise RuntimeError("Drift Client already initialized")
 
         state_public_key = get_state_public_key(self.program_id)
 
@@ -78,11 +48,9 @@ class Admin(ClearingHouse):
                     "admin": self.authority,
                     "state": state_public_key,
                     "quote_asset_mint": usdc_mint,
-                    "drift_signer": get_clearing_house_signer_public_key(
-                        self.program_id
-                    ),
-                    "rent": SYSVAR_RENT_PUBKEY,
-                    "system_program": SYS_PROGRAM_ID,
+                    "drift_signer": get_drift_client_signer_public_key(self.program_id),
+                    "rent": RENT,
+                    "system_program": ID,
                     "token_program": TOKEN_PROGRAM_ID,
                 },
             ),
@@ -92,18 +60,19 @@ class Admin(ClearingHouse):
 
     async def initialize_perp_market(
         self,
-        price_oracle: PublicKey,
+        market_index: int,
+        price_oracle: Pubkey,
         base_asset_reserve: int,
         quote_asset_reserve: int,
         periodicity: int,
         peg_multiplier: int = PEG_PRECISION,
-        oracle_source: OracleSource = OracleSource.PYTH(),
+        oracle_source: OracleSource = OracleSource.Pyth(),
         margin_ratio_initial: int = 2000,
         margin_ratio_maintenance: int = 500,
         liquidation_fee: int = 0,
         active_status: bool = True,
         name: list = [0] * 32,
-    ) -> TransactionSignature:
+    ) -> Signature:
         state_public_key = get_state_public_key(self.program.program_id)
         state = await get_state_account(self.program)
         market_pubkey = get_perp_market_public_key(
@@ -112,6 +81,7 @@ class Admin(ClearingHouse):
         )
 
         return await self.program.rpc["initialize_perp_market"](
+            market_index,
             base_asset_reserve,
             quote_asset_reserve,
             periodicity,
@@ -128,20 +98,20 @@ class Admin(ClearingHouse):
                     "state": state_public_key,
                     "oracle": price_oracle,
                     "perp_market": market_pubkey,
-                    "rent": SYSVAR_RENT_PUBKEY,
-                    "system_program": SYS_PROGRAM_ID,
+                    "rent": RENT,
+                    "system_program": ID,
                 }
             ),
         )
 
     async def initialize_spot_market(
         self,
-        mint: PublicKey,
+        mint: Pubkey,
         optimal_utilization: int = SPOT_RATE_PRECISION // 2,
         optimal_rate: int = SPOT_RATE_PRECISION,
         max_rate: int = SPOT_RATE_PRECISION,
-        oracle: PublicKey = PublicKey([0] * PublicKey.LENGTH),
-        oracle_source: OracleSource = OracleSource.QUOTE_ASSET(),
+        oracle: Pubkey = Pubkey([0] * Pubkey.LENGTH),
+        oracle_source: OracleSource = OracleSource.QuoteAsset(),
         initial_asset_weight: int = SPOT_WEIGHT_PRECISION,
         maintenance_asset_weight: int = SPOT_WEIGHT_PRECISION,
         initial_liability_weight: int = SPOT_WEIGHT_PRECISION,
@@ -153,14 +123,14 @@ class Admin(ClearingHouse):
     ):
         state_public_key = get_state_public_key(self.program_id)
         state = await get_state_account(self.program)
-        spot_index = state.number_of_spot_markets
+        spot_market_index = state.number_of_spot_markets
 
-        spot_public_key = get_spot_market_public_key(self.program_id, spot_index)
+        spot_public_key = get_spot_market_public_key(self.program_id, spot_market_index)
         spot_vault_public_key = get_spot_market_vault_public_key(
-            self.program_id, spot_index
+            self.program_id, spot_market_index
         )
         insurance_vault_public_key = get_insurance_fund_vault_public_key(
-            self.program_id, spot_index
+            self.program_id, spot_market_index
         )
 
         return await self.program.rpc["initialize_spot_market"](
@@ -183,13 +153,11 @@ class Admin(ClearingHouse):
                     "spot_market": spot_public_key,
                     "spot_market_vault": spot_vault_public_key,
                     "insurance_fund_vault": insurance_vault_public_key,
-                    "drift_signer": get_clearing_house_signer_public_key(
-                        self.program_id
-                    ),
+                    "drift_signer": get_drift_client_signer_public_key(self.program_id),
                     "spot_market_mint": mint,
                     "oracle": oracle,
-                    "rent": SYSVAR_RENT_PUBKEY,
-                    "system_program": SYS_PROGRAM_ID,
+                    "rent": RENT,
+                    "system_program": ID,
                     "token_program": TOKEN_PROGRAM_ID,
                 }
             ),
@@ -227,6 +195,42 @@ class Admin(ClearingHouse):
             ),
         )
 
+    async def update_perp_market_oracle(
+        self, market_index: int, oracle: Pubkey, oracle_source: OracleSource
+    ):
+        return await self.program.rpc["update_perp_market_oracle"](
+            oracle,
+            oracle_source,
+            ctx=Context(
+                accounts={
+                    "admin": self.authority,
+                    "state": get_state_public_key(self.program_id),
+                    "perp_market": get_perp_market_public_key(
+                        self.program_id, market_index
+                    ),
+                    "oracle": oracle,
+                }
+            ),
+        )
+
+    async def update_spot_market_oracle(
+        self, market_index: int, oracle: Pubkey, oracle_source: OracleSource
+    ):
+        return await self.program.rpc["update_spot_market_oracle"](
+            oracle,
+            oracle_source,
+            ctx=Context(
+                accounts={
+                    "admin": self.authority,
+                    "state": get_state_public_key(self.program_id),
+                    "spot_market": get_spot_market_public_key(
+                        self.program_id, market_index
+                    ),
+                    "oracle": oracle,
+                }
+            ),
+        )
+
     async def update_perp_market_max_fill_reserve_fraction(
         self,
         market_index: int,
@@ -518,62 +522,111 @@ class Admin(ClearingHouse):
         )
 
     async def update_k(
-        self, 
+        self,
         sqrt_k: int,
         perp_market_index: int,
     ):
-       return await self.send_ixs(
-            await self.update_k_ix(sqrt_k, perp_market_index)
-       ) 
-    
+        return await self.send_ixs(await self.update_k_ix(sqrt_k, perp_market_index))
+
     async def update_k_ix(
-        self, 
+        self,
         sqrt_k: int,
         perp_market_index: int,
     ):
-        market = await get_perp_market_account(
-            self.program, perp_market_index
-        )
+        market = await get_perp_market_account(self.program, perp_market_index)
 
         return self.program.instruction["update_k"](
-            sqrt_k, 
+            sqrt_k,
             ctx=Context(
                 accounts={
                     "admin": self.authority,
                     "state": get_state_public_key(self.program_id),
-                    "perp_market": get_perp_market_public_key(self.program_id, perp_market_index),
+                    "perp_market": get_perp_market_public_key(
+                        self.program_id, perp_market_index
+                    ),
                     "oracle": market.amm.oracle,
                 }
-            )
+            ),
         )
-    
+
     async def repeg_curve(
-        self, 
+        self,
         peg: int,
         perp_market_index: int,
     ):
-        return await self.send_ixs(
-            await self.repeg_curve_ix(peg, perp_market_index)
-        )
+        return await self.send_ixs(await self.repeg_curve_ix(peg, perp_market_index))
 
     async def repeg_curve_ix(
-        self, 
+        self,
         peg: int,
         perp_market_index: int,
     ):
-        market = await get_perp_market_account(
-            self.program, perp_market_index
-        )
+        market = await get_perp_market_account(self.program, perp_market_index)
 
         return self.program.instruction["repeg_amm_curve"](
-            peg, 
+            peg,
             ctx=Context(
                 accounts={
                     "admin": self.authority,
                     "state": get_state_public_key(self.program_id),
-                    "perp_market": get_perp_market_public_key(self.program_id, perp_market_index),
+                    "perp_market": get_perp_market_public_key(
+                        self.program_id, perp_market_index
+                    ),
                     "oracle": market.amm.oracle,
                 }
-            )
+            ),
+        )
+
+    async def update_initial_percent_to_liquidate(
+        self, initial_percent_to_liquidate: int
+    ) -> Signature:
+        state_public_key = get_state_public_key(self.program.program_id)
+
+        return await self.program.rpc["update_initial_pct_to_liquidate"](
+            initial_percent_to_liquidate,
+            ctx=Context(accounts={"admin": self.authority, "state": state_public_key}),
+        )
+
+    async def initialize_prelaunch_oracle(
+        self,
+        perp_market_index: int,
+        price: Optional[int] = None,
+        max_price: Optional[int] = None,
+    ):
+        params = PrelaunchOracleParams(perp_market_index, price, max_price)
+
+        return await self.program.rpc["initialize_prelaunch_oracle"](
+            params,
+            ctx=Context(
+                accounts={
+                    "admin": self.authority,
+                    "state": get_state_public_key(self.program_id),
+                    "prelaunch_oracle": get_prelaunch_oracle_public_key(
+                        self.program_id, perp_market_index
+                    ),
+                    "rent": RENT,
+                    "system_program": ID,
+                }
+            ),
+        )
+
+    async def update_prelaunch_oracle_params(
+        self,
+        perp_market_index: int,
+        price: Optional[int] = None,
+        max_price: Optional[int] = None,
+    ):
+        params = PrelaunchOracleParams(perp_market_index, price, max_price)
+
+        return await self.program.rpc["update_prelaunch_oracle_params"](
+            params,
+            ctx=Context(
+                accounts={
+                    "admin": self.authority,
+                    "state": get_state_public_key(self.program_id),
+                    "prelaunch_oracle": get_prelaunch_oracle_public_key(
+                        self.program_id, perp_market_index
+                    ),
+                }
+            ),
         )
-    
